package ru.dbotthepony.mc.otm.block.entity.decorative

import net.minecraft.core.BlockPos
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.world.MenuProvider
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.block.entity.SynchronizedBlockEntity
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.menu.decorative.HoloSignMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MBlocks

class HoloSignBlockEntity(blockPos: BlockPos, blockState: BlockState) : SynchronizedBlockEntity(MBlockEntities.HOLO_SIGN, blockPos, blockState), MenuProvider {
	var text by synchronizer.string("", name = "text", setter = { value, access, remote ->
		setChanged()
		access.write(value)
	})

	var locked = false

	override fun createMenu(p_39954_: Int, p_39955_: Inventory, p_39956_: Player): AbstractContainerMenu {
		return HoloSignMenu(p_39954_, p_39955_, this)
	}

	override fun getDisplayName(): Component {
		return MBlocks.HOLO_SIGN.name
	}

	override fun saveAdditional(p_187471_: CompoundTag) {
		super.saveAdditional(p_187471_)
		p_187471_[TEXT_KEY] = text
	}

	override fun load(p_155245_: CompoundTag) {
		super.load(p_155245_)
		locked = p_155245_.getBoolean(LOCKED_KEY)

		if (locked) {
			text = p_155245_.getString(TEXT_KEY)
		} else {
			text = truncate(p_155245_.getString(TEXT_KEY))
		}
	}

	companion object {
		const val TEXT_KEY = "SignText"
		const val LOCKED_KEY = "Locked"
		const val DEFAULT_MAX_NEWLINES = 8
		const val DEFAULT_MAX_LINE_LENGTH = 15
		private val NEWLINES = Regex("\r?\n")

		fun truncate(input: String): String {
			val lines = input.split(NEWLINES)
			val result = ArrayList<String>(lines.size.coerceAtMost(DEFAULT_MAX_NEWLINES))

			for (i in 0 until lines.size.coerceAtMost(DEFAULT_MAX_NEWLINES)) {
				if (lines[i].length > DEFAULT_MAX_LINE_LENGTH) {
					result.add(lines[i].substring(0, DEFAULT_MAX_LINE_LENGTH))
				} else {
					result.add(lines[i])
				}
			}

			return result.joinToString("\n")
		}
	}
}
